#========================================================================================
# package      DE_EPAGES::Amazon::API::Object::RemoteAWS
# state        public
#----------------------------------------------------------------------------------------
# description  base class for AWS remote adapter family
#========================================================================================
package DE_EPAGES::Amazon::API::Object::RemoteAWS;

use warnings;
use strict;

use DateTime;
use DE_EPAGES::Core::API::Log qw ( GetCommunicationLog LogDebug );
use Digest::SHA qw ( hmac_sha256_base64 );
use Digest::MD5 qw ( md5_base64 );
use HTTP::Request::Common qw ( POST );
use DE_EPAGES::Core::API::Error qw ( Error );
use LWP::UserAgent;
use URI;
use URI::Escape;
use Encode qw (encode);
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::ThirdPartyConfig::API::ThirdPartyConfig qw (GetDefaultTimeOut);

sub ErrorDefinition {
    return {
        'MissingConstructorArgument' =>
          'Need to set argument "#Argument" when calling new.',
    };
}

sub new {
    my $class = shift;
    my %args  = @_;
    my $self  = bless {}, $class;

    # obligatory: AWSURL AWSAccessKeyId AWSSecretKeyId
    for (qw/AWSURL AWSAccessKeyId AWSSecretKeyId/) {
        $self->{$_} = $args{$_} or Error( 'MissingConstructorArgument', { 'Argument' => $_ } );
    }

    # optional: logging
    $self->{'is_logging'} = $args{'Logging'} if defined $args{'Logging'};
    $self->{'awsUrl'} = URI->new( $args{'AWSURL'} );

    # injecting userAgent.
    if ( !$args{'userAgent'} ) {
        my $UserAgent = LWP::UserAgent->new;
        $UserAgent->timeout(GetDefaultTimeOut);
        $UserAgent->agent( 'ePages6 ' . LoadRootObject()->get('ProviderID') . ' ' . $UserAgent->agent() );
        $args{'userAgent'} = $UserAgent;
    }
    $self->{'userAgent'} = $args{'userAgent'};
    return $self;
}

sub _buildUrlQuery {
    my $self = shift;
    my ( $url, $request ) = @_;
    $url //= $self->awsUrl;
    $url = $url->clone();
    $url->query_form(%$request);
    return $url;
}
##no critic (ProhibitMagicNumbers,ProhibitFunctionSize)
# workaround: The AWS can not handle UFT-8 Widecharsigns like %C3%A4. Single Values like %2C are allowed
sub replaceUmlaute{
    my $self = shift;
    my ($request) = @_;
    my %umlaute = (
                    # a
                    chr(0x00E0) => 'a',
                    chr(0x00E1) => 'a',
                    chr(0x00E2) => 'a',
                    chr(0x00E3) => 'a',
                    chr(0x00E4) => 'ae',
                    chr(0x00E5) => 'a',
                    chr(0x00C0) => 'A',
                    chr(0x00C1) => 'A',
                    chr(0x00C2) => 'A',
                    chr(0x00C3) => 'A',
                    chr(0x00C4) => 'Ae',
                    chr(0x00C5) => 'A',
                    # e
                    chr(0x00E8) => 'e',
                    chr(0x00E9) => 'e',
                    chr(0x00EA) => 'e',
                    chr(0x00EB) => 'e',
                    chr(0x00C8) => 'E',
                    chr(0x00C9) => 'E',
                    chr(0x00CA) => 'E',
                    chr(0x00CB) => 'E',
                    # i
                    chr(0x00EC) => 'i',
                    chr(0x00ED) => 'i',
                    chr(0x00EE) => 'i',
                    chr(0x00EF) => 'i',
                    chr(0x00CC) => 'I',
                    chr(0x00CD) => 'I',
                    chr(0x00CE) => 'I',
                    chr(0x00CF) => 'I',
                    # o
                    chr(0x00F2) => 'o',
                    chr(0x00F3) => 'o',
                    chr(0x00F4) => 'o',
                    chr(0x00F5) => 'o',
                    chr(0x00F6) => 'oe',
                    chr(0x00F8) => 'o',
                    chr(0x00D2) => 'O',
                    chr(0x00D3) => 'O',
                    chr(0x00D4) => 'O',
                    chr(0x00D5) => 'O',
                    chr(0x00D6) => 'Oe',
                    chr(0x00D8) => 'O',
                    # u
                    chr(0x00F9) => 'u',
                    chr(0x00FA) => 'u',
                    chr(0x00FB) => 'u',
                    chr(0x00FC) => 'ue',
                    chr(0x00FD) => 'y',
                    chr(0x00FF) => 'y',
                    chr(0x00D9) => 'U',
                    chr(0x00DA) => 'U',
                    chr(0x00DB) => 'U',
                    chr(0x00DC) => 'Ue',
                    chr(0x00DD) => 'Y',
                    chr(0x00DF) => 'U',
                    # rest
                    chr(0x00df) => 'ss',
                    chr(0x00FE) => 'p',
                    );
    my $umlautkeys = join ('|', keys(%umlaute));
    foreach my $Key (keys %$request) {
        next unless defined $request->{$Key} ;
        $request->{$Key} =~ s/($umlautkeys)/$umlaute{$1}/g;
    }
}
## use critic

sub query {
    my $self = shift;
    my ( $request, $url ) = @_;

    my $rContent = delete $request->{rContent};
    $self->sign( $request, $url );
    $url = $self->_buildUrlQuery( $url, $request );
    my $UserAgent = $self->userAgent();
    my $Request = POST( $url->as_string() );
    if ( ref $rContent ) {
        my $Content = encode('utf8',$$rContent);
        $Request->content_type('text/xml; charset=utf-8');
        $Request->header( 'Content-MD5'    => $self->_signContent(\$Content) );
        $Request->header( 'Content-Length' => length($Content));
        $Request->content($Content);
    }
    my $Response = $UserAgent->request($Request);

    if ( $self->is_logging ) {
        my $Log = GetCommunicationLog('Amazon::'. ($self->loggerKey));
        if ( $Log->is_debug ) {
            $Log->debug( 'Request:' . $Request->as_string );
            $Log->debug( 'Response:' . $Response->as_string );
        }
    }

    return $Response;
}


sub urlencode {
    my $self      = shift;
    my ($request) = @_;
    my @StringArr = ();
    foreach my $key ( sort keys %$request ) {
        push @StringArr, $key . '=' . URI::Escape::uri_escape_utf8( $request->{$key}, '^A-Za-z0-9\-_.~' );
    }
    return join '&', @StringArr;
}

sub sign {
    my $self = shift;
    my ( $request, $url ) = @_;
    $url //= $self->awsUrl;

    $request->{'AWSAccessKeyId'}   //= $self->awsAccessKeyId;
    $request->{'Timestamp'}        //= DateTime->from_epoch( epoch => time() )->datetime() . 'Z';
    $request->{'SignatureVersion'} //= 2;
    $request->{'SignatureMethod'}  //= 'HmacSHA256';
    $request->{'Version'}          //= '2009-01-01';

    my $HTTPVerb                     = 'POST';
    my $HTTPRequestURI               = $url->path;
    my $ValueOfHostHeaderInLowercase = lc $url->host;
    my $StringToSign                 = "$HTTPVerb\n$ValueOfHostHeaderInLowercase\n$HTTPRequestURI\n" . $self->urlencode($request);

    if ( $self->is_logging ) {
        LogDebug( q{GetString:},      $StringToSign );
        LogDebug( q{AWSSecretKeyId:}, $self->awsSecretKeyId );
    }

    my $signature = hmac_sha256_base64( $StringToSign, $self->awsSecretKeyId );

    $self->_fillStringToMod4Length( \$signature, '=' );
    $request->{'Signature'} = $signature;
}

sub _fillStringToMod4Length {
    my $self = shift;
    my ( $rString, $char ) = @_;
    $char //= '=';
    $$rString .= $char x ((-length($$rString)) % 4 );
}

sub _signContent {
    my $self       = shift;
    my ($rContent) = @_;
    my $signature  = md5_base64($$rContent);
    $self->_fillStringToMod4Length(\$signature, '=');
    return $signature;
}

sub is_logging {                                    ## no critic (RequireArgUnpacking)
    return $_[0]->{'is_logging'};
}

sub setLogging {
    my $self = shift;
    my ($IsLogging) = @_;
    $self->{'is_logging'} = $IsLogging;
}

sub awsAccessKeyId {                                ## no critic (RequireArgUnpacking)
    return $_[0]->{'AWSAccessKeyId'};
}

sub awsSecretKeyId {                                ## no critic (RequireArgUnpacking)
    return $_[0]->{'AWSSecretKeyId'};
}

sub awsUrl {                                        ## no critic (RequireArgUnpacking)
    return $_[0]->{'awsUrl'};
}

sub userAgent {                                     ## no critic (RequireArgUnpacking)
    return $_[0]->{'userAgent'};
}

sub loggerKey {
    return 'RemoteAWS';
}

1;
