#========================================================================================
# package      DE_EPAGES::CustomForm::API::Cartridge
# base         DE_EPAGES::Design::API::DesignInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::CustomForm::API::Cartridge;
use base DE_EPAGES::ShopMailType::API::ShopMailTypeInstaller;

use strict;
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Dictionary::API::XML::TranslationImportHandler;
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObjectByPath Transaction );
use DE_EPAGES::Object::API::Table::Language qw ( GetAllInfoLanguages );
use DE_EPAGES::Object::API::Template qw (ProcessFileTemplate);
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw ( DeleteObjects );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::CustomForm::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::CustomForm.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/CustomForm',
        'Version'            => '7.78.0',
        'Patches'            => ['6.15.1','6.15.2','6.15.3','6.15.4','6.16.0','6.16.1','6.16.2','6.16.3','6.17.0','6.17.1','6.17.2','6.17.3','6.17.4','6.17.5','6.17.6','6.17.7','6.17.8','6.17.9','6.17.9.1','6.17.9.2','6.17.9.3','6.17.10','6.17.11','6.17.12','6.17.12.1','6.17.12.2','6.17.13','6.17.13.1','6.17.13.2','6.17.14','6.17.14.1','6.17.15','6.17.15.1','6.17.16','6.17.17','6.17.17.1','6.17.18','6.17.19','6.17.20','6.17.20.1','6.17.21','6.17.21.1','6.17.22','6.17.23','6.17.24','6.17.24.1','6.17.25','6.17.26','6.17.26.1','6.17.27','6.17.27.1','6.17.28','6.17.28.1','6.17.29','6.17.29.1','6.17.30','6.17.31','6.17.32','6.17.33','6.17.33.1','6.17.34','6.17.34.1','6.17.35','6.17.36','6.17.36.1','6.17.37','6.17.38','6.17.38.1','6.17.39','6.17.39.1','6.17.40','6.17.40.1','6.17.41','6.17.41.1','6.17.42','6.17.43','6.17.43.1','6.17.44','6.17.45','6.17.45.1','6.17.46','6.17.47','6.17.47.1','6.17.48','6.17.48.1','6.17.49','6.17.49.1','6.17.50','6.17.51','6.17.51.1','6.17.52','6.17.53','7.0.0','7.1.0','7.2.0','7.3.0','7.4.0','7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0','7.15.0','7.16.0','7.17.0','7.18.0','7.19.0','7.20.0','7.21.0','7.22.0','7.23.0','7.24.0','7.25.0','7.26.0','7.27.0','7.28.0','7.29.0','7.30.0','7.31.0','7.32.0','7.33.0','7.34.0','7.35.0','7.36.0','7.37.0','7.38.0','7.39.0','7.40.0','7.41.0','7.42.0','7.43.0','7.44.0','7.45.0','7.46.0','7.47.0','7.48.0','7.49.0','7.50.0','7.51.0','7.52.0','7.53.0','7.54.0','7.55.0','7.56.0','7.57.0','7.58.0','7.59.0','7.60.0','7.61.0','7.62.0','7.63.0','7.64.0','7.65.0','7.66.0','7.67.0','7.68.0','7.69.0','7.70.0','7.71.0','7.72.0','7.73.0','7.74.0','7.75.0','7.76.0','7.77.0'],
    );
    return bless $self, $class;
}

#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    $self->startInstall;
    $self->SUPER::install;

    if (ExistsCartridgeByPackage('DE_EPAGES::ContactForm')) {
        my $dbi = GetCurrentDBHandle();
        my $Log = GetLog();

        Transaction( sub {
            # insert CustomForm object
            my $Class = LoadClassByAlias('CustomForm');
            my $ClassID = $Class->id;
            $Log->info('insert CustomForm object in object table');
            $dbi->do(<<'END_SQL', [ $ClassID ]);
INSERT INTO object (classid, parentid, alias, position, guid, inherit, creationdate, siteid)
SELECT ?, oc.objectid, "ContactForm", 0, UPPER(UUID()), 1, NOW(), s.shopid
FROM shop s INNER JOIN object oc
ON oc.parentid = s.shopid
AND oc.alias = "Categories"
WHERE oc.objectid NOT IN (
    SELECT parentid
    FROM object
    WHERE alias = "ContactForm"
)
END_SQL
            $Log->info('insert CustomForm object in contentobject table');
            $dbi->do(<<END_SQL , [ $ClassID ]);
INSERT INTO contentobject (contentobjectid, isvisible, visibleinnavigation, siteid)
SELECT objectid, 0, 0, siteid
FROM object
WHERE classid = ?
END_SQL

            # insert Fields object at new CustomForm object
            my $ObjectClassID = LoadClassByAlias('Object')->id;
            $Log->info('insert Fields object in object table');
            $dbi->do(<<END_SQL , [ $ObjectClassID, $ClassID ]);
INSERT INTO object (classid, parentid, alias, position, guid, inherit, creationdate, siteid)
SELECT ?, objectid, "Fields", 10, UPPER(UUID()), 1, NOW(), siteid
FROM object
WHERE classid = ?
END_SQL

            # get translation hash
            my $hTranslation = _TranslationXMLToPerl($Log);

            # insert form fields
            ## no critic qw(ProhibitMagicNumbers)
            _InsertFormField('Subject',   'TextFormField',     10, $hTranslation, $dbi, $Log);
            _InsertFormField('Message',   'TextAreaFormField', 20, $hTranslation, $dbi, $Log);
            _InsertFormField('FirstName', 'TextFormField',     30, $hTranslation, $dbi, $Log);
            _InsertFormField('LastName',  'TextFormField',     40, $hTranslation, $dbi, $Log);
            _InsertFormField('Phone',     'TextFormField',     50, $hTranslation, $dbi, $Log);
            _InsertFormField('EMail',     'EMailFormField',    60, $hTranslation, $dbi, $Log);
            ## use critic

            # set translation for CustomForm shop attributes
            my $Path = 'Categories/ContactForm';
            foreach my $hLanguage (grep { $_->{'IsUsed'} } @{GetAllInfoLanguages()}) {
                _SaveLocalizedAttributesByPath($Path, $hTranslation, $hLanguage, $Class, 'ContactForm', $dbi, $Log);
            }

            # translate and render MailSubject and ConfirmMessage
            my $hTranslationFiles = {
                'MailSubject'    => $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/CustomForm/Templates/MBO/MailSubjectContent.txt',
                'ConfirmMessage' => $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/CustomForm/Templates/MBO/ConfirmMessageContent.txt'
            };
            $Log->info('insert translated attributes MailSubject and ConfirmMessage in localizedstringattribute table');
            foreach my $hLanguage (grep { $_->{'IsUsed'} } @{GetAllInfoLanguages()}) {
                local $DE_EPAGES::Object::API::Language::LANGUAGEID = $hLanguage->{'LanguageID'};
                foreach my $Attribute (qw( MailSubject ConfirmMessage )) {
                    my $TranslatedAttribute = ProcessFileTemplate($hTranslationFiles->{$Attribute}, $hLanguage->{'Code2'}, {}) // '';
                    my $TranslatedAttributeID = $Class->attribute($Attribute)->id;
                    $dbi->do(<<END_SQL , [ $TranslatedAttributeID, $hLanguage->{'LanguageID'}, $TranslatedAttribute, $ClassID, $hLanguage->{'LanguageID'}]);
INSERT INTO localizedstringattribute (objectid, attributeid, languageid, value)
SELECT objectid, ?, ?, ?
FROM object o INNER JOIN shoplanguage sl
ON sl.shopid = o.siteid
WHERE o.classid = ?
AND alias = "ContactForm"
AND sl.languageid = ?
END_SQL
                }
            }

            # copy current recipient addresses (if defined)
            my $ToEmailAttributeID = $Class->attribute('ToEMail')->id;
            $Log->info('copy TO recipient address into stringattribute table');
            $dbi->do(<<END_SQL , [ $ToEmailAttributeID, $ClassID ] );
INSERT INTO stringattribute (objectid, attributeid, value)
SELECT co.objectid, ?, m.mailto
FROM mailtype m
INNER JOIN object mo ON m.mailtypeid = mo.objectid
INNER JOIN object co ON co.siteid = mo.siteid
WHERE mo.alias = "ShopContactForm"
AND co.alias = "ContactForm"
AND co.classid = ?
AND m.mailto IS NOT NULL
END_SQL

            my $CcEmailAttributeID = $Class->attribute('CcEMail')->id;
            $Log->info('copy CC recipient address into stringattribute table');
            $dbi->do(<<END_SQL , [ $CcEmailAttributeID, $ClassID ] );
INSERT INTO stringattribute (objectid, attributeid, value)
SELECT co.objectid, ?, m.cc
FROM mailtype m
INNER JOIN object mo ON m.mailtypeid = mo.objectid
INNER JOIN object co ON co.siteid = mo.siteid
WHERE mo.alias = "ShopContactForm"
AND co.alias = "ContactForm"
AND co.classid = ?
AND m.cc IS NOT NULL
END_SQL

            my $BccEmailAttributeID = $Class->attribute('BccEMail')->id;
            $Log->info('copy BCC recipient address into stringattribute table');
            $dbi->do(<<END_SQL , [ $BccEmailAttributeID, $ClassID ] );
INSERT INTO stringattribute (objectid, attributeid, value)
SELECT co.objectid, ?, m.bcc
FROM mailtype m
INNER JOIN object mo ON m.mailtypeid = mo.objectid
INNER JOIN object co ON co.siteid = mo.siteid
WHERE mo.alias = "ShopContactForm"
AND co.alias = "ContactForm"
AND co.classid = ?
AND m.bcc IS NOT NULL
END_SQL

            # replace "link to contact form"
            my $LinkElementClass = LoadObjectByPath('/NavElements/LinkElement');
            my $ContactFormLinkClass = LoadObjectByPath('/NavElements/ContactFormLink');
            $Log->info('update classid of ContactFormLink objects to LinkElement');
            $dbi->do(<<END_SQL , [ $LinkElementClass->id, $ContactFormLinkClass->id ] );
UPDATE object
SET classid = ?
WHERE classid = ?
END_SQL

            $Log->info('update navelementid of ContactFormLink objects to LinkElement');
            $dbi->do(<<END_SQL , [ $LinkElementClass->id, $ContactFormLinkClass->id ] );
UPDATE navbarelement
SET navelementid = ?
WHERE navelementid = ?
END_SQL

            # set Target
            my $TargetAttributeID = $LinkElementClass->attribute('Target')->id;
            $Log->info('insert target attribute at referenceattribute table');
            $dbi->do(<<END_SQL , [ $TargetAttributeID, $Class->id, $LinkElementClass->id ]);
INSERT INTO referenceattribute (objectid, attributeid, referencedobjectid)
SELECT lo.objectid, ?, cfo.objectid
FROM object lo
INNER JOIN object cfo ON lo.siteid = cfo.siteid AND cfo.classid = ?
WHERE lo.classid = ?
AND lo.alias LIKE 'ContactFormLink%'
END_SQL

            # set Text (localized)
            $Log->info('insert translated attribute Text in localizedstringattribute table');
            foreach my $hLanguage (grep { $_->{'IsUsed'} } @{GetAllInfoLanguages()}) {
                my $Text = $hTranslation->{$hLanguage->{'Code2'}}->{'Categories/ContactForm'}->{'NavigationCaption'} // '';
                my $AttributeID = $LinkElementClass->attribute('Text')->id;
                $dbi->do(<<END_SQL , [ $AttributeID, $hLanguage->{'LanguageID'}, $Text, $LinkElementClass->id, $hLanguage->{'LanguageID'}]);
INSERT INTO localizedstringattribute (objectid, attributeid, languageid, value)
SELECT o.objectid, ?, ?, ?
FROM object o
INNER JOIN shoplanguage sl ON sl.shopid = o.siteid
WHERE o.classid = ?
AND o.alias LIKE 'ContactFormLink%'
AND sl.languageid = ?
END_SQL
            }

            # deinstall old ContactForm cartridge
            $Log->info('delete DE_EPAGES::ContactForm cartridge');
            {
                local $ENV{'EPAGES_BACKUP_DIR'} = '';

                #get installer
                $self->loadInstaller('DE_EPAGES::ContactForm')->uninstall;
            }
        });
    }

    ImportObjects($self->xmlDirectory . '/MailTypes.xml');
    $self->finishInstall;
    return;
}

sub _InsertFormField {
    my ($Alias, $ClassAlias, $Position, $hTranslation, $dbi, $Log) = @_;
    my $FormFieldClass = LoadClassByAlias($ClassAlias);
    my $CustomFormClass = LoadClassByAlias('CustomForm');

    # insert form field object
    $Log->info("insert form field $Alias of type $ClassAlias in object table");
    $dbi->do(<<END_SQL , [ $FormFieldClass->id, $Alias, $Position, $CustomFormClass->id ]);
INSERT INTO object (classid, parentid, alias, position, guid, inherit, creationdate, siteid)
SELECT ?, o2.objectid, ?, ?, UPPER(UUID()), 1, NOW(), o1.siteid
FROM
 object o1
JOIN object o2 ON o1.objectid = o2.parentid
WHERE o1.classid = ? AND o2.alias = 'Fields'
END_SQL

    # make all form fields editable, except for the e-mail address
    $Log->info('set IsAlterable form field attribute to 1 (expect for EMail) in booleanattribute table');
    if( $FormFieldClass ne 'EMailFormField' ) {
        my $IsAlterableAttribute = $FormFieldClass->attribute('IsAlterable');
        $dbi->do(<<END_SQL , [ $IsAlterableAttribute->id, $FormFieldClass->id, $Alias ]);
INSERT INTO booleanattribute (objectid, attributeid, value)
SELECT objectid, ?, 1
FROM object
WHERE classid = ?
AND alias = ?
END_SQL
    }

    # set all form fields mandatory, except for the phone number field
    $Log->info('set IsMandatory form field attribute to 1 (expect for Phone) in booleanattribute table');
    if( $Alias ne 'Phone' ) {
        my $IsMandatoryAttribute = $FormFieldClass->attribute('IsMandatory');
        $dbi->do(<<END_SQL , [ $IsMandatoryAttribute->id, $FormFieldClass->id, $Alias ]);
INSERT INTO booleanattribute (objectid, attributeid, value)
SELECT objectid, ?, 1
FROM object
WHERE classid = ?
AND alias = ?
END_SQL
    }

    # translate form field attributes
    $Log->info('insert translated form field attributes in localizedstringattribute table');
    foreach my $hLanguage (grep { $_->{'IsUsed'} } @{GetAllInfoLanguages()}) {
        my $Path = "Categories/ContactForm/Fields/$Alias";
        _SaveLocalizedAttributesByPath($Path, $hTranslation, $hLanguage, $FormFieldClass, $Alias, $dbi);
    }
}

sub _SaveLocalizedAttributesByPath {
    my ($Path, $hTranslation, $hLanguage, $FormFieldClass, $Alias, $dbi) = @_;
    my $hTranslatedAttributes = $hTranslation->{$hLanguage->{'Code2'}}->{$Path};
    foreach my $TranslatedAttributeAlias (keys %$hTranslatedAttributes) {
        my $TranslatedAttribute = $hTranslatedAttributes->{$TranslatedAttributeAlias} // '';
        my $TranslatedAttributeID = $FormFieldClass->attribute($TranslatedAttributeAlias)->id;
        $dbi->do(<<END_SQL , [ $TranslatedAttributeID, $TranslatedAttribute, $FormFieldClass->id, $Alias, $hLanguage->{'LanguageID'} ]);
INSERT INTO localizedstringattribute (objectid, attributeid, languageid, value)
SELECT objectid, ?, sl.languageid, ?
FROM object o
INNER JOIN shoplanguage sl ON sl.shopid = o.siteid
WHERE o.classid = ?
AND o.alias = ?
AND sl.languageid = ?
END_SQL
    }
}

sub _TranslationXMLToPerl {
    my ($Log) = @_;
    $Log->info('map translation xml to perl hash');
    my @LanguageCodes = map { $_->{Code2} } grep { $_->{'IsUsed'} } @{GetAllInfoLanguages()};
    my $hStructure;
    foreach my $Code2 ( @LanguageCodes  ) {
        my $File = $ENV{'EPAGES_CARTRIDGES'} . "/DE_EPAGES/ShopConfiguration/Data/Private/ShopImport/Translation.Basic.$Code2.xml";
        my $hStruct = DE_EPAGES::Dictionary::API::XML::TranslationImportHandler::ImportTranslation($File);
        $hStructure->{$Code2} = $hStruct->{$Code2};
    }
    return $hStructure;
}

#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;
    $self->call(sub { DeleteObjects($self->xmlDirectory .'/MailTypes.xml') });
    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

sub patch_6_15_0 {
    my $self = shift;

    $self->importPatchDir('Patch_6_15_0', 'Patch');

    my $dbi = GetCurrentDBHandle();

    GetLog()->info( 'EPG-26640: make e-mail fields in custom forms non-editable' );
    my $Class = LoadClassByAlias('EMailFormField');
    my $Attribute = $Class->attribute('IsAlterable');
    $dbi->do( <<'END_SQL', [$Attribute->id, $Class->id] );
DELETE FROM booleanattribute
WHERE attributeid = ?
AND objectid IN (
  SELECT objectid
  FROM object
  WHERE classid = ?
)
END_SQL

    # EPG-26640: set all form fields mandatory, except for the phone number field
    GetLog()->info('EPG-26640: set all form fields mandatory, except for the phone number field');
    my $CustomFormFieldClass = LoadClassByAlias('CustomFormField');
    my $TextFormFieldClass = LoadClassByAlias('TextFormField');
    my $TextAreaFormFieldClass = LoadClassByAlias('TextAreaFormField');
    my $EMailFormFieldClass = LoadClassByAlias('EMailFormField');
    my $IsMandatoryAttribute = $CustomFormFieldClass->attribute('IsMandatory');
    $dbi->do(<<END_SQL , [ $IsMandatoryAttribute->id, $TextFormFieldClass->id, $TextAreaFormFieldClass->id, $EMailFormFieldClass->id, $IsMandatoryAttribute->id ]);
INSERT INTO booleanattribute (objectid, attributeid, value)
SELECT objectid, ?, 1
FROM object
WHERE classid IN (?, ?, ?)
AND alias NOT LIKE "Phone"
AND objectid NOT IN (
    SELECT objectid
    FROM booleanattribute
    WHERE attributeid = ?
)
END_SQL
    
    return '6.15.1';
}

1;
