#========================================================================================
# package      DE_EPAGES::MolliePayments::API::Requests::Methods
# state        public
# description  Helper module for generating MolliePayments Methods requests to epagesj
#========================================================================================
package DE_EPAGES::MolliePayments::API::Requests::Methods;
use base DE_EPAGES::MolliePayments::API::Requests::BaseRequest;

use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::MolliePayments::API::Tools qw ( GetSupportedLocale );
use DE_EPAGES::MolliePayments::API::Constants qw ( 
    METHODS_PARAM_ORDERS_RESOURCE
    METHODS_PARAM_INCLUDE_WALLETS
);

use URI;
use URI::QueryParam;

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::MolliePayments::API::Requests::Methods->new($hMethodsData);
#----------------------------------------------------------------------------------------
# description  MolliePayments Methods requests to epagesj helper constructor
#----------------------------------------------------------------------------------------
# input        $hMethodsData | required data to generate Methods requests to epagesj | hash.ref
# return       $Object | new object | DE_EPAGES::MolliePayments::API::Requests::Methods
#========================================================================================
sub new {
    my $class = shift;
    my ($hMethodsData) = @_;

    my $self = $class->SUPER::new($hMethodsData);

    return $self;
}

#========================================================================================
# function     withProfileId
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->withProfileId($ProfileId);
#----------------------------------------------------------------------------------------
# description  Set profile id as part of the payload
#----------------------------------------------------------------------------------------
# return       $Object | new object | DE_EPAGES::MolliePayments::API::Requests::Methods
#========================================================================================
sub withProfileId {
    my $self = shift;
    my ($ProfileId) = @_;

    $self->{'queryParams'}->{'profileId'} = $ProfileId;

    return $self;
}

sub path {
    my $self = shift;
    my ($Path) = @_;

    $self->{'path'} = $Path;

    return $self;
}

#========================================================================================
# function     getEpagesjRequestSuffixUrlPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Url = $MethodsRequest->getEpagesjRequestSuffixUrlPath( $HttpRequestMethod );
#----------------------------------------------------------------------------------------
# description  It returns the Methods postfix url path from the epagesj endpoint
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $Url | Methods suffix url path from the epagesj endpoint | string
#========================================================================================
sub getEpagesjRequestSuffixUrlPath {
    my $self = shift;
    my ($HttpRequestMethod) = @_;

    if ($HttpRequestMethod ne 'GET') {
        Error( 'Only GET HttpRequestMethod accepted in Methods handler', { Parameter => 'HttpRequestMethod' } );
    }

    if (defined $self->{'path'} && $self->{'path'} eq 'ideal') {
        return '/methods/ideal';
    }

    return '/methods';
}

#========================================================================================
# function     getHttpHeaders
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestHeader = $self->getHttpHeaders( $HttpRequestMethod );
#----------------------------------------------------------------------------------------
# description  It returns required headers of the Methods request
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $hRequestHeader | Methods request headers hash | hash.ref
#========================================================================================
sub getHttpHeaders {
    my $self = shift;
    my ($HttpRequestMethod) = @_;

    if ($HttpRequestMethod eq 'GET') {
        return {
            'X-AccessToken' => $self->{'data'}->{'payment'}->get('AccessToken')
        };
    }
}

#========================================================================================
# function     getQueryParams
# state        public
#----------------------------------------------------------------------------------------
# syntax       $QueryParams = $self->getQueryParams();
#----------------------------------------------------------------------------------------
# description  It returns the query Params of the Methods request
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $QueryParams | Methods request $QueryParams string | string
#========================================================================================
sub getQueryParams {
    my $self = shift;

    $self->{'queryParams'}->{'locale'} = $self->_getMerchantLocale();

    if (defined $self->{'path'} && $self->{'path'} eq 'ideal') {
        $self->{'queryParams'}->{'include'} = 'issuers';
        return $self->{'queryParams'};
    }

    #We want to get payment methods that use the createOrders flow
    $self->{'queryParams'}->{'resource'} = METHODS_PARAM_ORDERS_RESOURCE;

    $self->{'queryParams'}->{'includeWallets'} = METHODS_PARAM_INCLUDE_WALLETS;

    my $ShopCountryCode = $self->_getShopCountryCode();
    if (isValid($ShopCountryCode)) {
        $self->{'queryParams'}->{'billingCountry'} = $ShopCountryCode;
    }

    return $self->{'queryParams'};
}

#========================================================================================
sub _getMerchantLocale {
    my $self = shift;

    my $LocaleID = $self->_getMerchantLocaleId();

    return GetSupportedLocale($LocaleID);
}

#========================================================================================
sub _getMerchantLocaleId {
    my $self = shift;

    my $Shop = $self->getShop();
    my $ShopAddress = $Shop->get('Address');
    my $ShopEmail = $ShopAddress->get('EMail');
    my @MerchantUsers = ();
    my @SuperMerchantUsers = ();

    foreach my $User (@{$self->_getShopUsers($Shop)}) {

        return $User->get('LocaleID') if ($User->get('EMail') eq $ShopEmail);

        push @SuperMerchantUsers, $User if ($User->get('IsSuperMerchant'));
        push @MerchantUsers, $User if ($User->get('IsMerchant'));
    }

    if (scalar(@SuperMerchantUsers) == 1) {
        my $User = pop @SuperMerchantUsers;
        return $User->get('LocaleID');
    }

    if (scalar(@MerchantUsers) == 1) {
        my $User = pop @MerchantUsers;
        return $User->get('LocaleID');
    }

    return $Shop->get('LocaleID');
}

#========================================================================================
sub _getShopCountryCode {
    my $self = shift;

    my $Shop = $self->getShop();

    my $ShopAddress = $Shop->get('Address');
    return unless defined( $ShopAddress );

    my $ShopCountry = $ShopAddress->get('Country');
    return unless defined( $ShopCountry );

    return $ShopCountry->{'Code2'};
}
#========================================================================================
sub _getShopUsers {
    my $self = shift;
    my ($Shop) = @_;

    return $Shop->folder('Users')->children;
}

1;
