#========================================================================================
# package      DE_EPAGES::MolliePayments::API::Requests::Orders
# state        public
# description  Helper module for generating MolliePayments orders requests to epagesj
#========================================================================================
package DE_EPAGES::MolliePayments::API::Requests::Orders;
use base DE_EPAGES::MolliePayments::API::Requests::BaseRequest;

use DateTime;
use DE_EPAGES::MolliePayments::API::Tools qw ( FormatFloatForMollie GetSupportedLocale );
use DE_EPAGES::MolliePayments::API::Constants qw ( SUPPORTED_ORDER_EMBED );
use DE_EPAGES::Core::API::Error qw (Error);

use Math::Round;

use strict;

use constant POST_HTTP_METHOD => 'POST';
use constant GET_HTTP_METHOD => 'GET';
use constant DELETE_HTTP_METHOD => 'DELETE';
use constant PUT_HTTP_METHOD => 'PUT';

use constant UNARY_TO_PERCENTAGE_FACTOR => 100;

use constant INITIAL_ORDER_NUMBER_NOT_SET => '-';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::MolliePayments::API::Requests::Orders->new( );
#----------------------------------------------------------------------------------------
# description  MolliePayments orders requests to epagesj helper constructor
#----------------------------------------------------------------------------------------
# input        $hOrdersData | required data to generate order requests to epagesj | hash.ref
# return       $Object | new object | DE_EPAGES::MolliePayments::API::Requests::Orders
#========================================================================================
sub new {
    my $class = shift;
    my ($hOrdersData) = @_;

    my $self = $class->SUPER::new($hOrdersData);

    return $self;
}


#========================================================================================
# function     getEpagesjRequestSuffixUrlPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Url = $OrdersRequest->getEpagesjRequestSuffixUrlPath( $HttpRequestMethod );
#----------------------------------------------------------------------------------------
# description  It returns the orders postfix url path from the epagesj endpoint
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $Url | orders suffix url path from the epagesj endpoint | string
#========================================================================================
sub getEpagesjRequestSuffixUrlPath {
    my $self = shift;
    my ($HttpRequestMethod) = @_;

    if ($HttpRequestMethod eq POST_HTTP_METHOD) {
        return '/orders';
    }
    if ($HttpRequestMethod eq GET_HTTP_METHOD) {
        return '/orders/' . $self->{'urlPath'}->{'orderId'};
    }
    if ($HttpRequestMethod eq DELETE_HTTP_METHOD) {
        return '/orders/' . $self->{'urlPath'}->{'orderId'};
    }
    if ($HttpRequestMethod eq PUT_HTTP_METHOD) {
        return '/orders/' . $self->{'urlPath'}->{'orderId'};
    }
}

#========================================================================================
# function     getHttpHeaders
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestHeader = $self->getHttpHeaders( $HttpRequestMethod );
#----------------------------------------------------------------------------------------
# description  It returns required headers of the orders request
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $hRequestHeader | orders request headers hash | hash.ref
#========================================================================================
sub getHttpHeaders {
    my $self = shift;
    my ($HttpRequestMethod) = @_;
        return {
            'X-AccessToken' => $self->{'data'}->{'payment'}->get('AccessToken')
        };
}

#========================================================================================
# function     getQueryParams
# state        public
#----------------------------------------------------------------------------------------
# syntax       $QueryParams = $self->getQueryParams();
#----------------------------------------------------------------------------------------
# description  It returns the query Params of the Methods request
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $QueryParams | Methods request $QueryParams string | string
#========================================================================================
sub getQueryParams {
    my $self = shift;
    my ($HttpRequestMethod) = @_;

    if ($HttpRequestMethod eq GET_HTTP_METHOD) {
        return $self->{'queryParams'};
    }

    if ($HttpRequestMethod eq DELETE_HTTP_METHOD) {
        return $self->{'queryParams'};
    }

    Error('getQueryParams is not implemented for:' . $HttpRequestMethod);
}

#========================================================================================
# function     getPayload
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hPayload = $self->getPayload( $HttpRequestMethod );
#----------------------------------------------------------------------------------------
# description  It returns the body payload of the orders request
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $hPayload | order request payload hash | hash.ref
#========================================================================================
sub getPayload {
    my $self = shift;
    my ($HttpRequestMethod) = @_;

    if ($HttpRequestMethod eq POST_HTTP_METHOD) {
        my $Payment = $self->{'data'}->{'payment'};
        
        $self->{'payload'}->{'profileId'} = $Payment->get('ProfileId');
        $self->{'payload'}->{'method'} = $Payment->get('Method');
        $self->{'payload'}->{'testmode'} = $Payment->get('IsInTestMode') ? 'true' : 'false';

        my $Basket = $self->{'data'}->{'basket'};
        my $LineItemContainer = $Basket->get('LineItemContainer');
        $self->{'payload'}->{'amount'} = $self->_getBasketTotalAmount($LineItemContainer);
        $self->{'payload'}->{'locale'} = GetSupportedLocale($LineItemContainer->get('LocaleID'));
        $self->{'payload'}->{'orderNumber'} = INITIAL_ORDER_NUMBER_NOT_SET;

        $self->{'payload'}->{'redirectUrl'} = $self->{'data'}->{'redirectUrl'};
        $self->{'payload'}->{'webhookUrl'} = $self->{'data'}->{'orderWebhookUrl'};
        $self->{'payload'}->{'lines'} = $self->_getBasketLineItems($LineItemContainer);
        $self->{'payload'}->{'billingAddress'} = $self->_getAddress($Basket->get('BillingAddress'));

        my $ShippingAddress = $Basket->get('ShippingAddress');
        if ($ShippingAddress) {
            $self->{'payload'}->{'shippingAddress'} = $self->_getAddress($ShippingAddress);
        }

        $self->{'payload'}->{'payment'} = {
            'webhookUrl' => $self->{'data'}->{'paymentWebhookUrl'},
        };

        # payload-payment handling for ideal
        if (defined $self->{'data'}->{'idealIssuer'}) {
            $self->{'payload'}->{'payment'} = {
                'webhookUrl' => $self->{'data'}->{'paymentWebhookUrl'},
                'issuer'     => $self->{'data'}->{'idealIssuer'},
            };
        }
    }

    return $self->{'payload'};
}

#========================================================================================
# function     withOrderId
# state        public
#----------------------------------------------------------------------------------------
# syntax       $OrdersRequest->withOrderId( $HttpRequestMethod );
#----------------------------------------------------------------------------------------
# description  It sets the url path for OrderId
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $self | order request | object
#========================================================================================
sub withOrderId {
    my $self = shift;
    my ($OrderId) = @_;

    $self->{'urlPath'}->{'orderId'} = $OrderId;
    return $self;
}

#========================================================================================
# function     withOrderNumber
# state        public
#----------------------------------------------------------------------------------------
# syntax       $OrdersRequest->withOrderNumber( $OrderNumber );
#----------------------------------------------------------------------------------------
# description  It sets the url path for OrderId
#----------------------------------------------------------------------------------------
# input        $OrderNumber | Order number string | string
# return       $self | order request | object
#========================================================================================
sub withOrderNumber {
    my $self = shift;
    my ($OrderNumber) = @_;

    $self->{'payload'}->{'orderNumber'} = $OrderNumber;
    return $self;
}

#========================================================================================
# function     withEmbed
# state        public
#----------------------------------------------------------------------------------------
# syntax       $OrdersRequest->withEmbed( $HttpRequestMethod );
#----------------------------------------------------------------------------------------
# description  It sets the url path for Embed
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $self | order request | object
#========================================================================================
sub withEmbed {
    my $self = shift;
    my ($Embed) = @_;

    if ($Embed ~~ SUPPORTED_ORDER_EMBED) {
        $self->{'queryParams'}->{'embed'} = $Embed;
    }
    return $self;
}

#========================================================================================
# function     handleResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hFilteredResponse = $self->handleResponse($hResponse);
#----------------------------------------------------------------------------------------
# description  It returns the response filtered
#----------------------------------------------------------------------------------------
# input        $hResponse | response hash | hash.ref
# return       $hMollieOrder | filtered response hash | hash.ref
#========================================================================================
sub handleResponse {
    my $self = shift;
    my ($hResponse) = @_;

    my $hMollieOrder = {
        'id'     => $hResponse->{'id'},
        'status' => $hResponse->{'status'},
        'mode'   => $hResponse->{'mode'},
        'amount' => $hResponse->{'amount'}
    };

    if(defined($hResponse->{'_links'}) && defined($hResponse->{'_links'}->{'checkout'}) ) {
        $hMollieOrder->{'checkout'} = $hResponse->{'_links'}->{'checkout'};
    } else { # this order request may need order lines data
        $hMollieOrder->{'lines'} = $hResponse->{'lines'};
    }

    if(defined($hResponse->{'_embedded'}) &&
        defined($hResponse->{'_embedded'}->{'payments'}) ) {
        $hMollieOrder->{'payments'} = $hResponse->{'_embedded'}->{'payments'};
    }

    return $hMollieOrder;
}

#========================================================================================
# function     getLineItemRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hLineItemRequest = getLineItemRequest($LineItem, $CurrencyID);
#----------------------------------------------------------------------------------------
# description  Generate line item request data
#----------------------------------------------------------------------------------------
# input        $LineItem | line item object | object
# input        $hContainerInfo | container data like currency | hash.ref
# return       $hLineItemRequest | Mollie line item request hash | hash.ref
#========================================================================================
sub getLineItemRequest {
    my $self = shift;
    my ($LineItem, $hContainerInfo) = @_;

    my $hLineItemInfo = $LineItem->get([qw(Alias Quantity BasePrice LineItemPrice Discount TaxRate)]);

    $hLineItemInfo->{'Discount'} //= 0;
    $hLineItemInfo->{'Quantity'} = $self->_getQuantityAsUnits($LineItem, $hLineItemInfo);
    $hLineItemInfo->{'BasePrice'} = $self->_reckonUnitPrice($hLineItemInfo);
    my $Name = $LineItem->get('NameOrAlias', $hContainerInfo->{'LanguageID'});

    return {
        'name' => $LineItem->instanceOf('LineItemTax') ? "tax($Name)" : $Name,
        'quantity' => $hLineItemInfo->{'Quantity'},
        'unitPrice' => {
            'value' => FormatFloatForMollie($hLineItemInfo->{'BasePrice'}),
            'currency' => $hContainerInfo->{'CurrencyID'}
        },
        'totalAmount' => {
            'value' => FormatFloatForMollie($hLineItemInfo->{'LineItemPrice'}),
            'currency' => $hContainerInfo->{'CurrencyID'}
        },
        'discountAmount' => {
            'value' => FormatFloatForMollie($hLineItemInfo->{'Discount'}),
            'currency' => $hContainerInfo->{'CurrencyID'}
        },
        'vatRate' => FormatFloatForMollie($hLineItemInfo->{'TaxRate'} * UNARY_TO_PERCENTAGE_FACTOR),
        'vatAmount' => {
            'value' => $self->_calculateVatAmount($hLineItemInfo),
            'currency' => $hContainerInfo->{'CurrencyID'}
        },
        'metadata' => {
            'alias' => $hLineItemInfo->{'Alias'}
        }
    };

}


#========================================================================================
# private methods
#========================================================================================
sub _getBasketTotalAmount {
    my $self = shift;
    my ($LineItemContainer) = @_;

    return {
        'value' => FormatFloatForMollie($LineItemContainer->get('GrandTotal')),
        'currency' => $LineItemContainer->get('CurrencyID')
    };
}

#----------------------------------------------------------------------------------------
sub _getBasketLineItems {
    my $self = shift;
    my ($LineItemContainer) = @_;

    my $hContainerInfo = $LineItemContainer->get([ qw( CurrencyID LanguageID ) ]);
    my @LineItems = ();
    foreach my $LineItem (@{$LineItemContainer->children()}) {
        if ( !$LineItem->instanceOf('LineItemPaymentMolliePayments') ) {
            my $LineItem = $self->getLineItemRequest($LineItem, $hContainerInfo);
            push(@LineItems, $LineItem);
        }
    }

    return \@LineItems;
}

#----------------------------------------------------------------------------------------
sub _getQuantityAsUnits {
    my $self = shift;
    my ($LineItem, $hLineItemInfo) = @_;

    my $QuantityAsUnits = 1;
    if ($LineItem->instanceOf('LineItemProduct')) {
        if ($hLineItemInfo->{'Quantity'} == round($hLineItemInfo->{'Quantity'})) {
            $QuantityAsUnits = $hLineItemInfo->{'Quantity'};
        }
    }

    return $QuantityAsUnits;
}

#----------------------------------------------------------------------------------------
sub _reckonUnitPrice {
    my $self = shift;
    my ($hLineItemInfo) = @_;

    return ($hLineItemInfo->{'LineItemPrice'} + $hLineItemInfo->{'Discount'}) / $hLineItemInfo->{'Quantity'};
}

#----------------------------------------------------------------------------------------
sub _calculateVatAmount {
    my $self = shift;
    my ($hLineItemInfo) = @_;

    my $TotalAmount = $hLineItemInfo->{'LineItemPrice'};
    my $VatRate = $hLineItemInfo->{'TaxRate'} * UNARY_TO_PERCENTAGE_FACTOR;
    my $VatAmount = $TotalAmount * ($VatRate / (UNARY_TO_PERCENTAGE_FACTOR + $VatRate));

    return FormatFloatForMollie($VatAmount);
}

#----------------------------------------------------------------------------------------
sub _getAddress {
    my $self = shift;
    my ($BasketAddress) = @_;

    my $hAddress = {
        'givenName' => $BasketAddress->get('FirstName'),
        'familyName' => $BasketAddress->get('LastName'),
        'email' => $BasketAddress->get('EMail'),
        'streetAndNumber' => $BasketAddress->get('Street'),
        'postalCode' => $BasketAddress->get('Zipcode'),
        'city' => $BasketAddress->get('City'),
        'country' => defined($BasketAddress->get('Country')) ? $BasketAddress->get('Country')->{'Code2'} : undef,
    };

    my $State = $BasketAddress->get('State');
    if ($State) {
        $hAddress->{'region'} = $State;
    }

    return $hAddress;
}

1;
