use strict;
use feature 'say';

use Getopt::Long;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadObjectByPath );
use HTTP::Status qw( HTTP_CONFLICT );

sub Main {

    my $Password;
    my $Help;
    my $ShopAlias;
    my $StoreName;
    my $Verbose = 1;
    my $LocaleID ;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'shop=s'      => \$ShopAlias,
        'output!'     => \$Verbose,
        'locale=s'    => \$LocaleID,
    );

    usage() if $Help;

    unless ($StoreName) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    unless ($ShopAlias) {
        print STDERR "Missing parameter -shopalias.\n\n";
        usage();
    }
    unless ($LocaleID) {
        print STDERR "Missing parameter -locale.\n\n";
        usage();
    }

    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub   => sub {
            # get all attibutes from shop without a deeper structure
            my $Shop = LoadObjectByPath('/Shops')->child($ShopAlias);
            eval {
                my $DefaultLanguageID = $Shop->get('LanguageID');
                my $ShopName = $Shop->get('NameOrAlias', $DefaultLanguageID);
                # inform Unity about the new language (new unity shop is created)
                # get the locale id first wich will be added to the locales
                # the unity shop is created with this locale id and the name
                # Add a new visible locale ignoring HTTP_CONFLICT in case the
                # locale already exists
                $Shop->sendUnityRequest('POST', '/shop/locale', undef, {
                    'title'  => $ShopName,
                    'isVisible' => JSON::true,
                    'locale' => $LocaleID}, [HTTP_CONFLICT]);
                # Set new locale to be the default
                $Shop->sendUnityRequest('PATCH', '/shop', undef, {'locale' => $LocaleID}, [HTTP_CONFLICT]);
            };
            if (ExistsError()) {
                my $Error = GetError();
                LogError($Error);
                if ($Verbose) {
                    say 'Synchronizing the shop attributes to Unity failed.';
                    say 'Error message: ' . $Error->message;
                }
            } else {
                say 'Adding locale to the epages-ui for this shop was successful.' if $Verbose;
            }
        }
    );
}

sub usage {
    print <<END_USAGE;
Description:
    Adds a locale to a given shop in epages-ui and set it to default
    for the related Unity shop.

Usage:
    perl $0 [options] [flags]

options:
    -passwd         database user password     (optional)
    -storename      the name of the Store      (required)
    -shop           the shopalias of the shop  (required)
    -locale         the new locale of the shop (required)
flags:
    -help           show the command line options
    -nooutput       do not print any output

Example:
    perl $0 -storename <Store> -shop <ShopAlias> -locale de_DE
END_USAGE
    exit 2;
}

RunScript('Sub' => \&Main);


